#**************************************************************************
#   Copyright (C) 2006 by Michael Andreen <harvATruinDOTnu>               *
#                                                                         *
#   This program is free software; you can redistribute it and/or modify  *
#   it under the terms of the GNU General Public License as published by  *
#   the Free Software Foundation; either version 2 of the License, or     *
#   (at your option) any later version.                                   *
#                                                                         *
#   This program is distributed in the hope that it will be useful,       *
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#   GNU General Public License for more details.                          *
#                                                                         *
#   You should have received a copy of the GNU General Public License     *
#   along with this program; if not, write to the                         *
#   Free Software Foundation, Inc.,                                       *
#   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
#**************************************************************************/
package NDIRC::CommonStates;

use strict;
use warnings;
use feature ':5.10';

use POE::Session;
use POE::Component::IRC::Plugin::Logger;
use POE::Component::IRC::Plugin::BotTraffic;
use POE::Component::IRC::Plugin::Connector;

use NDIRC::Dispatcher;

# We registered for all events, this will produce some debug info.
sub _default {
	my ($event, $args) = @_[ARG0 .. $#_];
	my @output = ( "$event: " );

	for my $arg (@$args) {
		if ( ref $arg eq 'ARRAY' ) {
			push( @output, '[' . join(', ', @$arg ) . ']' );
		}
		else {
			push ( @output, "'$arg'" );
		}
	}
	print join ' ', @output, "\n";
	return 0;
}

sub _start {
	my ($kernel,$heap,$session) = @_[KERNEL,HEAP,SESSION];

	# retrieve our component's object from the heap where we stashed it
	my $irc = $heap->{irc};
	$kernel->sig( DIE => 'sig_DIE' );
	$kernel->sig( USR1 => 'sig_usr1' );
	$kernel->sig( USR2 => 'sig_usr2' );
	$kernel->sig( INT => 'signal_handler' );

	$irc->plugin_add( 'BotTraffic', POE::Component::IRC::Plugin::BotTraffic->new() );
	$irc->plugin_add('Logger', POE::Component::IRC::Plugin::Logger->new(
		Path    => 'irclogs',
		DCC     => 0,
		Private => 1,
		Public  => 1,
		Sort_by_date => 1,
		Strip_color => 1,
		Strip_formatting => 1,
	));

	$heap->{connector} = POE::Component::IRC::Plugin::Connector->new();
	$irc->plugin_add( 'Connector' => $heap->{connector} );

	$kernel->signal($session => 'USR2');

	$irc->yield( register => 'all' );
	$irc->yield( connect => { } );

	$kernel->delay( refresh => 60 );
	return;
}

sub sig_usr1 {
	my $kernel = $_[KERNEL];
	$kernel->yield( 'refresh' );
}

sub sig_usr2 {
	my $heap = $_[HEAP];

	open COMMANDS, 'commands';
	my @commands = split /\W+/, do { local $/; <COMMANDS> };
	close COMMANDS;

	say "Loading commands from: @commands";
	my $disp = new NDIRC::Dispatcher;
	$disp->load(@commands);

	open CHANNELS, 'channels';
	while (<CHANNELS>){
		my ($chan, @types) = split /\s+/;
		say "$chan - @types";
		$disp->add_channel($chan,\@types);
	}
	close CHANNELS;

	$heap->{disp} = $disp;
}

sub sig_DIE {
	my( $kernel,$sig, $ex ) = @_[ KERNEL,ARG0, ARG1 ];
	say "DIED!!!!!!!!!!!!!!";
	# $sig is 'DIE'
	# $ex is the exception hash
	warn "$$: error in event: $ex->{error_str}";
	$kernel->sig_handled();

	# Send the signal to session that sent the original event.
	#if( $ex->{source_session} ne $_[SESSION] ) {
	#$kernel->signal( $ex->{source_session}, 'DIE', $sig, $ex );
	#}
}

sub signal_handler {
	my ($kernel, $signal_name, $heap) = @_[KERNEL, ARG0, HEAP];
	print "First session caught SIG$signal_name\n";

	given($signal_name){
		when ('INT') {
			exit unless $heap->{irc}->connected;
			$heap->{irc}->yield(quit => 'Bye!');
			$kernel->sig_handled();
		}
	}
	#$kernel->sig_handled();
}

sub irc_disconnected {
	my ($sender,$heap) = @_[SENDER,HEAP];

	exit;
}

sub irc_001 {
	my ($sender,$heap) = @_[SENDER,HEAP];

	# Since this is an irc_* event, we can get the component's object by
	# accessing the heap of the sender. Then we register and connect to the
	# specified server.
	my $irc = $sender->get_heap();

	print "Connected to ", $irc->server_name(), "\n";

	# we join our channels
	$irc->yield( join => $_ ) for grep /^#/, keys %{$heap->{disp}->channels};
	return;
}

sub irc_invite {
	my ($sender, $heap, $who, $channel) = @_[SENDER, HEAP, ARG0 .. ARG1];
	my $irc = $sender->get_heap();

	$irc->yield( join => $_ ) for grep /^$channel$/i, keys %{$heap->{disp}->channels}
}

1;
